@@ -116,7 +116,8 @@ namespace LCompilers {
 
         static inline bool is_aggregate_or_array_type(ASR::expr_t* var) {
             return (ASR::is_a<ASR::Struct_t>(*ASRUtils::expr_type(var)) ||
-                    ASRUtils::is_array(ASRUtils::expr_type(var)));
+                    ASRUtils::is_array(ASRUtils::expr_type(var)) ||
+                    ASR::is_a<ASR::SymbolicExpression_t>(*ASRUtils::expr_type(var)));
         }
 
         template <class Struct>
@@ -344,9 +345,15 @@ namespace LCompilers {
                         SymbolTable* temp_scope = current_scope;
 
                         if (asr_owner_sym && temp_scope->get_counter() != ASRUtils::symbol_parent_symtab(x.m_name)->get_counter() &&
-                            !ASR::is_a<ASR::AssociateBlock_t>(*asr_owner_sym) && !ASR::is_a<ASR::ExternalSymbol_t>(*x.m_name) &&
-                                !ASR::is_a<ASR::Variable_t>(*x.m_name)) {
-                            function_dependencies.push_back(al, ASRUtils::symbol_name(x.m_name));
+                            !ASR::is_a<ASR::ExternalSymbol_t>(*x.m_name) && !ASR::is_a<ASR::Variable_t>(*x.m_name)) {
+                            if (ASR::is_a<ASR::AssociateBlock_t>(*asr_owner_sym) || ASR::is_a<ASR::Block_t>(*asr_owner_sym)) {
+                                temp_scope = temp_scope->parent;
+                                if (temp_scope->get_counter() != ASRUtils::symbol_parent_symtab(x.m_name)->get_counter()) {
+                                    function_dependencies.push_back(al, ASRUtils::symbol_name(x.m_name));
+                                }
+                            } else {
+                                function_dependencies.push_back(al, ASRUtils::symbol_name(x.m_name));
+                            }    
                         }
                     }
                     if( ASR::is_a<ASR::ExternalSymbol_t>(*x.m_name) &&
@@ -367,11 +374,17 @@ namespace LCompilers {
                         }
 
                         SymbolTable* temp_scope = current_scope;
-
+                        
                         if (asr_owner_sym && temp_scope->get_counter() != ASRUtils::symbol_parent_symtab(x.m_name)->get_counter() &&
-                            !ASR::is_a<ASR::AssociateBlock_t>(*asr_owner_sym) && !ASR::is_a<ASR::ExternalSymbol_t>(*x.m_name) &&
-                                !ASR::is_a<ASR::Variable_t>(*x.m_name)) {
-                            function_dependencies.push_back(al, ASRUtils::symbol_name(x.m_name));
+                            !ASR::is_a<ASR::ExternalSymbol_t>(*x.m_name) && !ASR::is_a<ASR::Variable_t>(*x.m_name)) {
+                            if (ASR::is_a<ASR::AssociateBlock_t>(*asr_owner_sym) || ASR::is_a<ASR::Block_t>(*asr_owner_sym)) {
+                                temp_scope = temp_scope->parent;
+                                if (temp_scope->get_counter() != ASRUtils::symbol_parent_symtab(x.m_name)->get_counter()) {
+                                    function_dependencies.push_back(al, ASRUtils::symbol_name(x.m_name));
+                                }
+                            } else {
+                                function_dependencies.push_back(al, ASRUtils::symbol_name(x.m_name));
+                            }    
                         }
                     }
 
@@ -388,6 +401,7 @@ namespace LCompilers {
                 void visit_BlockCall(const ASR::BlockCall_t& x) {
                     SymbolTable *parent_symtab = current_scope;
                     ASR::Block_t* block = ASR::down_cast<ASR::Block_t>(x.m_m);
+                    current_scope = block->m_symtab;
                     for (size_t i=0; i<block->n_body; i++) {
                         visit_stmt(*(block->m_body[i]));
                     }
@@ -573,8 +587,9 @@ namespace LCompilers {
                                                     array_ref, idoloop->m_values[i], nullptr));
                     doloop_body.push_back(al, doloop_stmt);
                     if( arr_idx != nullptr ) {
+                        ASR::expr_t* one = ASRUtils::EXPR(ASR::make_IntegerConstant_t(al, arr_var->base.loc, 1, ASRUtils::TYPE(ASR::make_Integer_t(al, arr_var->base.loc, 4))));
                         ASR::expr_t* increment = ASRUtils::EXPR(ASR::make_IntegerBinOp_t(al, arr_var->base.loc,
-                                                    arr_idx, ASR::binopType::Add, const_1, ASRUtils::expr_type(arr_idx), nullptr));
+                                                    arr_idx, ASR::binopType::Add, one, ASRUtils::expr_type(arr_idx), nullptr));
                         ASR::stmt_t* assign_stmt = ASRUtils::STMT(ASR::make_Assignment_t(al, arr_var->base.loc,
                                                     arr_idx, increment, nullptr));
                         doloop_body.push_back(al, assign_stmt);
@@ -762,7 +777,7 @@ namespace LCompilers {
     }
 
     static inline void handle_fn_return_var(Allocator &al, ASR::Function_t *x,
-            bool (*is_array_or_struct)(ASR::expr_t*)) {
+            bool (*is_array_or_struct_or_symbolic)(ASR::expr_t*)) {
         if (ASRUtils::get_FunctionType(x)->m_abi == ASR::abiType::BindPython) {
             return;
         }
@@ -774,7 +789,7 @@ namespace LCompilers {
             * in avoiding deep copies and the destination memory directly gets
             * filled inside the function.
             */
-            if( is_array_or_struct(x->m_return_var)) {
+            if( is_array_or_struct_or_symbolic(x->m_return_var)) {
                 for( auto& s_item: x->m_symtab->get_scope() ) {
                     ASR::symbol_t* curr_sym = s_item.second;
                     if( curr_sym->type == ASR::symbolType::Variable ) {
@@ -813,7 +828,7 @@ namespace LCompilers {
         for (auto &item : x->m_symtab->get_scope()) {
             if (ASR::is_a<ASR::Function_t>(*item.second)) {
                 handle_fn_return_var(al, ASR::down_cast<ASR::Function_t>(
-                    item.second), is_array_or_struct);
+                    item.second), is_array_or_struct_or_symbolic);
             }
         }
     }
