@@ -12,6 +12,10 @@
 
 #if defined(_MSC_VER)
 #  include <winsock2.h>
+#  include <io.h>
+#  define ftruncate _chsize_s
+#else
+#  include <unistd.h>
 #endif
 
 #include <libasr/runtime/lfortran_intrinsics.h>
@@ -69,6 +73,14 @@ struct Stacktrace {
 
 #endif // HAVE_RUNTIME_STACKTRACE
 
+#ifdef HAVE_LFORTRAN_MACHO
+    #define INT64 "%lld"
+#elif HAVE_BUILD_TO_WASM
+    #define INT64 "%lld"
+#else
+    #define INT64 "%ld"
+#endif
+
 // This function performs case insensitive string comparison
 bool streql(const char *s1, const char* s2) {
 #if defined(_MSC_VER)
@@ -96,6 +108,27 @@ LFORTRAN_API void _lfortran_random_number(int n, double *v)
     }
 }
 
+LFORTRAN_API void _lfortran_init_random_seed(unsigned seed)
+{
+    srand(seed);
+}
+
+LFORTRAN_API void _lfortran_init_random_clock()
+{
+    unsigned int count;
+#if defined(_MSC_VER)
+    count = (unsigned int)clock();
+#else
+    struct timespec ts;
+    if (clock_gettime(CLOCK_MONOTONIC, &ts) == 0) {
+        count = (unsigned int)(ts.tv_nsec);
+    } else {
+        count = (unsigned int)clock();
+    }
+#endif
+    srand(count);
+}
+
 LFORTRAN_API double _lfortran_random()
 {
     return (rand() / (double) RAND_MAX);
@@ -174,8 +207,7 @@ void handle_integer(char* format, int val, char** result) {
             for (int i = 0; i < (min_width - len - sign_width); i++) {
                 *result = append_to_string(*result, "0");
             }
-        }
-        else {
+        } else {
             for (int i = 0; i < (width - len - sign_width); i++) {
                 *result = append_to_string(*result, " ");
             }
@@ -193,6 +225,18 @@ void handle_integer(char* format, int val, char** result) {
     }
 }
 
+void handle_logical(char* format, bool val, char** result) {
+    int width = atoi(format + 1);
+    for (int i = 0; i < width - 1; i++) {
+        *result = append_to_string(*result, " ");
+    }
+    if (val) {
+        *result = append_to_string(*result, "T");
+    } else {
+        *result = append_to_string(*result, "F");
+    }
+}
+
 void handle_float(char* format, double val, char** result) {
     int width = 0, decimal_digits = 0;
     long integer_part = (long)fabs(val);
@@ -285,6 +329,12 @@ void handle_decimal(char* format, double val, int scale, char** result, char* c)
         i--;
     }
 
+    int exp = 2;
+    char* exp_loc = strchr(num_pos, 'e');
+    if (exp_loc != NULL) {
+        exp = atoi(++exp_loc);
+    }
+
     char* ptr = strchr(val_str, '.');
     if (ptr != NULL) {
         memmove(ptr, ptr + 1, strlen(ptr));
@@ -299,7 +349,7 @@ void handle_decimal(char* format, double val, int scale, char** result, char* c)
         memmove(val_str, val_str + 1, strlen(val_str));
         decimal--;
     }
-    if (format[1] == 'S') {
+    if (tolower(format[1]) == 's') {
         scale = 1;
         decimal--;
     }
@@ -383,7 +433,7 @@ void handle_decimal(char* format, double val, int scale, char** result, char* c)
     if (atoi(num_pos) == 0) {
         sprintf(exponent, "%+02d", (integer_length > 0 && integer_part != 0 ? integer_length - scale : decimal));
     } else {
-        sprintf(exponent, "%+03d", (integer_length > 0 && integer_part != 0 ? integer_length - scale : decimal));
+        sprintf(exponent, "%+0*d", exp+1, (integer_length > 0 && integer_part != 0 ? integer_length - scale : decimal));
     }
 
     strcat(formatted_value, exponent);
@@ -422,6 +472,9 @@ char** parse_fortran_format(char* format, int *count, int *item_start) {
             case '/' :
                 format_values_2[format_values_count++] = substring(format, index, index+1);
                 break;
+            case '*' :
+                format_values_2[format_values_count++] = substring(format, index, index+1);
+                break;
             case '"' :
                 start = index++;
                 while (format[index] != '"') {
@@ -449,11 +502,20 @@ char** parse_fortran_format(char* format, int *count, int *item_start) {
             case 'd' :
             case 'e' :
             case 'f' :
+            case 'l' :
                 start = index++;
+                bool dot = false;
                 if(tolower(format[index]) == 's') index++;
                 while (isdigit(format[index])) index++;
-                if (format[index] == '.') index++;
+                if (format[index] == '.') {
+                    dot = true;
+                    index++;
+                }
                 while (isdigit(format[index])) index++;
+                if (dot && tolower(format[index]) == 'e') {
+                    index++;
+                    while (isdigit(format[index])) index++;
+                }
                 format_values_2[format_values_count++] = substring(format, start, index);
                 index--;
                 break;
@@ -464,12 +526,11 @@ char** parse_fortran_format(char* format, int *count, int *item_start) {
                 *item_start = format_values_count;
                 break;
             default :
-                if (isdigit(format[index]) && tolower(format[index+1]) == 'p') {
+                if (
+                    (format[index] == '-' && isdigit(format[index + 1]) && tolower(format[index + 2]) == 'p')
+                    || ((isdigit(format[index])) && tolower(format[index + 1]) == 'p')) {
                     start = index;
-                    if (index > 0 && format[index-1] == '-') {
-                        start = index - 1;
-                    }
-                    index = index + 1;
+                    index = index + 1 + (format[index] == '-');
                     format_values_2[format_values_count++] = substring(format, start, index + 1);
                 } else if (isdigit(format[index])) {
                     start = index;
@@ -497,6 +558,9 @@ char** parse_fortran_format(char* format, int *count, int *item_start) {
                         }
                         index--;
                     }
+                } else if (format[index] != ' ') {
+                    fprintf(stderr, "Unsupported or unrecognized `%c` in format string\n", format[index]);
+                    exit(1);
                 }
         }
         index++;
@@ -515,13 +579,14 @@ LFORTRAN_API char* _lcompilers_string_format_fortran(int count, const char* form
     modified_input_string[len] = '\0';
     if (format[0] == '(' && format[len-1] == ')') {
         memmove(modified_input_string, modified_input_string + 1, strlen(modified_input_string));
-        modified_input_string[len-1] = '\0';
+        modified_input_string[len-2] = '\0';
     }
     int format_values_count = 0,item_start_idx=0;
     char** format_values = parse_fortran_format(modified_input_string,&format_values_count,&item_start_idx);
     char* result = (char*)malloc(sizeof(char));
     result[0] = '\0';
     int item_start = 0;
+    bool array = false;
     while (1) {
         int scale = 0;
         for (int i = item_start; i < format_values_count; i++) {
@@ -555,6 +620,8 @@ LFORTRAN_API char* _lcompilers_string_format_fortran(int count, const char* form
 
             if (value[0] == '/') {
                 result = append_to_string(result, "\n");
+            } else if (value[0] == '*') {
+                array = true;
             } else if (isdigit(value[0]) && tolower(value[1]) == 'p') {
                 // Scale Factor nP
                 scale = atoi(&value[0]);
@@ -614,13 +681,23 @@ LFORTRAN_API char* _lcompilers_string_format_fortran(int count, const char* form
                 count--;
                 double val = va_arg(args, double);
                 handle_float(value, val, &result);
+            } else if (tolower(value[0]) == 'l') {
+                if ( count == 0 ) break;
+                count--;
+                char* val_str = va_arg(args, char*);
+                bool val = (strcmp(val_str, "True") == 0);
+                handle_logical(value, val, &result);
             } else if (strlen(value) != 0) {
+                if ( count == 0 ) break;
+                count--;
                 printf("Printing support is not available for %s format.\n",value);
             }
 
         }
         if ( count > 0 ) {
-            result = append_to_string(result, "\n");
+            if (!array) {
+                result = append_to_string(result, "\n");
+            }
             item_start = item_start_idx;
         } else {
             break;
@@ -1180,6 +1257,38 @@ LFORTRAN_API double_complex_t _lfortran_zatanh(double_complex_t x)
     return catanh(x);
 }
 
+// trunc -----------------------------------------------------------------------
+
+LFORTRAN_API float _lfortran_strunc(float x)
+{
+    return truncf(x);
+}
+
+LFORTRAN_API double _lfortran_dtrunc(double x)
+{
+    return trunc(x);
+}
+
+// fix -----------------------------------------------------------------------
+
+LFORTRAN_API float _lfortran_sfix(float x)
+{
+    if (x > 0.0) {
+        return floorf(x);
+    } else {
+        return ceilf(x);
+    }
+}
+
+LFORTRAN_API double _lfortran_dfix(double x)
+{
+    if (x > 0.0) {
+        return floor(x);
+    } else {
+        return ceil(x);
+    }
+}
+
 // phase --------------------------------------------------------------------
 
 LFORTRAN_API float _lfortran_cphase(float_complex_t x)
@@ -1222,10 +1331,10 @@ LFORTRAN_API void _lfortran_strcpy(char** x, char *y, int8_t free_target)
         if (*x) {
             free((void *)*x);
         }
-        *x = (char *) malloc(strlen(y)*sizeof(char));
-        _lfortran_string_init(strlen(y)+1, *x);
     }
-    for (size_t i = 0; i < strlen(*x); i ++) {
+    *x = (char*) malloc((strlen(y) + 1) * sizeof(char));
+    _lfortran_string_init(strlen(y) + 1, *x);
+    for (size_t i = 0; i < strlen(*x); i++) {
         if (i < strlen(y)) {
             x[0][i] = y[i];
         } else {
@@ -1771,13 +1880,13 @@ LFORTRAN_API void _lfortran_cpu_time(double *t) {
 
 LFORTRAN_API void _lfortran_i32sys_clock(
         int32_t *count, int32_t *rate, int32_t *max) {
-#if defined(_MSC_VER) || defined(__MACH__)
+#if defined(_MSC_VER)
         *count = - INT_MAX;
         *rate = 0;
         *max = 0;
 #else
     struct timespec ts;
-    if(clock_gettime(CLOCK_MONOTONIC, &ts) == -1) {
+    if(clock_gettime(CLOCK_MONOTONIC, &ts) == 0) {
         *count = (int32_t)(ts.tv_nsec / 1000000) + ((int32_t)ts.tv_sec * 1000);
         *rate = 1e3; // milliseconds
         *max = INT_MAX;
@@ -1791,13 +1900,13 @@ LFORTRAN_API void _lfortran_i32sys_clock(
 
 LFORTRAN_API void _lfortran_i64sys_clock(
         uint64_t *count, int64_t *rate, int64_t *max) {
-#if defined(_MSC_VER) || defined(__MACH__)
+#if defined(_MSC_VER)
         *count = - INT_MAX;
         *rate = 0;
         *max = 0;
 #else
     struct timespec ts;
-    if(clock_gettime(CLOCK_MONOTONIC, &ts) == -1) {
+    if(clock_gettime(CLOCK_MONOTONIC, &ts) == 0) {
         *count = (uint64_t)(ts.tv_nsec) + ((uint64_t)ts.tv_sec * 1000000000);
         // FIXME: Rate can be in microseconds or nanoseconds depending on
         //          resolution of the underlying platform clock.
@@ -1830,12 +1939,10 @@ LFORTRAN_API double _lfortran_time()
 }
 
 LFORTRAN_API void _lfortran_sp_rand_num(float *x) {
-    srand(time(0));
     *x = rand() / (float) RAND_MAX;
 }
 
 LFORTRAN_API void _lfortran_dp_rand_num(double *x) {
-    srand(time(0));
     *x = rand() / (double) RAND_MAX;
 }
 
@@ -1883,6 +1990,7 @@ void store_unit_file(int32_t unit_num, FILE* filep, bool unit_file_bin) {
 }
 
 FILE* get_file_pointer_from_unit(int32_t unit_num, bool *unit_file_bin) {
+    *unit_file_bin = false;
     for( int i = 0; i <= last_index_used; i++ ) {
         if( unit_to_file[i].unit == unit_num ) {
             *unit_file_bin = unit_to_file[i].unit_file_bin;
@@ -2067,7 +2175,7 @@ LFORTRAN_API void _lfortran_read_int32(int32_t *p, int32_t unit_num)
 {
     if (unit_num == -1) {
         // Read from stdin
-        scanf("%d", p);
+        (void)!scanf("%d", p);
         return;
     }
 
@@ -2079,9 +2187,9 @@ LFORTRAN_API void _lfortran_read_int32(int32_t *p, int32_t unit_num)
     }
 
     if (unit_file_bin) {
-        fread(p, sizeof(*p), 1, filep);
+        (void)!fread(p, sizeof(*p), 1, filep);
     } else {
-        fscanf(filep, "%d", p);
+        (void)!fscanf(filep, "%d", p);
     }
 }
 
@@ -2089,7 +2197,7 @@ LFORTRAN_API void _lfortran_read_int64(int64_t *p, int32_t unit_num)
 {
     if (unit_num == -1) {
         // Read from stdin
-        scanf("%lld", p);
+        (void)!scanf(INT64, p);
         return;
     }
 
@@ -2101,9 +2209,9 @@ LFORTRAN_API void _lfortran_read_int64(int64_t *p, int32_t unit_num)
     }
 
     if (unit_file_bin) {
-        fread(p, sizeof(*p), 1, filep);
+        (void)!fread(p, sizeof(*p), 1, filep);
     } else {
-        fscanf(filep, "%lld", p);
+        (void)!fscanf(filep, INT64, p);
     }
 }
 
@@ -2112,7 +2220,7 @@ LFORTRAN_API void _lfortran_read_array_int8(int8_t *p, int array_size, int32_t u
     if (unit_num == -1) {
         // Read from stdin
         for (int i = 0; i < array_size; i++) {
-            scanf("%s", &p[i]);
+            (void)!scanf("%s", &p[i]);
         }
         return;
     }
@@ -2125,10 +2233,10 @@ LFORTRAN_API void _lfortran_read_array_int8(int8_t *p, int array_size, int32_t u
     }
 
     if (unit_file_bin) {
-        fread(p, sizeof(int8_t), array_size, filep);
+        (void)!fread(p, sizeof(int8_t), array_size, filep);
     } else {
         for (int i = 0; i < array_size; i++) {
-            fscanf(filep, "%s", &p[i]);
+            (void)!fscanf(filep, "%s", &p[i]);
         }
     }
 }
@@ -2138,7 +2246,7 @@ LFORTRAN_API void _lfortran_read_array_int32(int32_t *p, int array_size, int32_t
     if (unit_num == -1) {
         // Read from stdin
         for (int i = 0; i < array_size; i++) {
-            scanf("%d", &p[i]);
+            (void)!scanf("%d", &p[i]);
         }
         return;
     }
@@ -2151,10 +2259,10 @@ LFORTRAN_API void _lfortran_read_array_int32(int32_t *p, int array_size, int32_t
     }
 
     if (unit_file_bin) {
-        fread(p, sizeof(int32_t), array_size, filep);
+        (void)!fread(p, sizeof(int32_t), array_size, filep);
     } else {
         for (int i = 0; i < array_size; i++) {
-            fscanf(filep, "%d", &p[i]);
+            (void)!fscanf(filep, "%d", &p[i]);
         }
     }
 }
@@ -2164,7 +2272,7 @@ LFORTRAN_API void _lfortran_read_char(char **p, int32_t unit_num)
     if (unit_num == -1) {
         // Read from stdin
         *p = (char*)malloc(strlen(*p) * sizeof(char));
-        scanf("%s", *p);
+        (void)!scanf("%s", *p);
         return;
     }
 
@@ -2178,9 +2286,9 @@ LFORTRAN_API void _lfortran_read_char(char **p, int32_t unit_num)
     int n = strlen(*p);
     *p = (char*)malloc(n * sizeof(char));
     if (unit_file_bin) {
-        fread(*p, sizeof(char), n, filep);
+        (void)!fread(*p, sizeof(char), n, filep);
     } else {
-        fscanf(filep, "%s", *p);
+        (void)!fscanf(filep, "%s", *p);
     }
     if (streql(*p, "")) {
         printf("Runtime error: End of file!\n");
@@ -2192,7 +2300,7 @@ LFORTRAN_API void _lfortran_read_float(float *p, int32_t unit_num)
 {
     if (unit_num == -1) {
         // Read from stdin
-        scanf("%f", p);
+        (void)!scanf("%f", p);
         return;
     }
 
@@ -2204,9 +2312,9 @@ LFORTRAN_API void _lfortran_read_float(float *p, int32_t unit_num)
     }
 
     if (unit_file_bin) {
-        fread(p, sizeof(*p), 1, filep);
+        (void)!fread(p, sizeof(*p), 1, filep);
     } else {
-        fscanf(filep, "%f", p);
+        (void)!fscanf(filep, "%f", p);
     }
 }
 
@@ -2215,7 +2323,7 @@ LFORTRAN_API void _lfortran_read_array_float(float *p, int array_size, int32_t u
     if (unit_num == -1) {
         // Read from stdin
         for (int i = 0; i < array_size; i++) {
-            scanf("%f", &p[i]);
+            (void)!scanf("%f", &p[i]);
         }
         return;
     }
@@ -2228,10 +2336,10 @@ LFORTRAN_API void _lfortran_read_array_float(float *p, int array_size, int32_t u
     }
 
     if (unit_file_bin) {
-        fread(p, sizeof(float), array_size, filep);
+        (void)!fread(p, sizeof(float), array_size, filep);
     } else {
         for (int i = 0; i < array_size; i++) {
-            fscanf(filep, "%f", &p[i]);
+            (void)!fscanf(filep, "%f", &p[i]);
         }
     }
 }
@@ -2241,7 +2349,7 @@ LFORTRAN_API void _lfortran_read_array_double(double *p, int array_size, int32_t
     if (unit_num == -1) {
         // Read from stdin
         for (int i = 0; i < array_size; i++) {
-            scanf("%lf", &p[i]);
+            (void)!scanf("%lf", &p[i]);
         }
         return;
     }
@@ -2254,10 +2362,10 @@ LFORTRAN_API void _lfortran_read_array_double(double *p, int array_size, int32_t
     }
 
     if (unit_file_bin) {
-        fread(p, sizeof(double), array_size, filep);
+        (void)!fread(p, sizeof(double), array_size, filep);
     } else {
         for (int i = 0; i < array_size; i++) {
-            fscanf(filep, "%lf", &p[i]);
+            (void)!fscanf(filep, "%lf", &p[i]);
         }
     }
 }
@@ -2269,7 +2377,7 @@ LFORTRAN_API void _lfortran_read_array_char(char **p, int array_size, int32_t un
         for (int i = 0; i < array_size; i++) {
             int n = 1; // TODO: Support character length > 1
             p[i] = (char*) malloc(n * sizeof(char));
-            scanf("%s", p[i]);
+            (void)!scanf("%s", p[i]);
         }
         return;
     }
@@ -2285,9 +2393,9 @@ LFORTRAN_API void _lfortran_read_array_char(char **p, int array_size, int32_t un
         int n = 1; // TODO: Support character length > 1
         p[i] = (char*) malloc(n * sizeof(char));
         if (unit_file_bin) {
-            fread(p[i], sizeof(char), n, filep);
+            (void)!fread(p[i], sizeof(char), n, filep);
         } else {
-            fscanf(filep, "%s", p[i]);
+            (void)!fscanf(filep, "%s", p[i]);
         }
     }
 }
@@ -2296,7 +2404,7 @@ LFORTRAN_API void _lfortran_read_double(double *p, int32_t unit_num)
 {
     if (unit_num == -1) {
         // Read from stdin
-        scanf("%lf", p);
+        (void)!scanf("%lf", p);
         return;
     }
 
@@ -2308,9 +2416,9 @@ LFORTRAN_API void _lfortran_read_double(double *p, int32_t unit_num)
     }
 
     if (unit_file_bin) {
-        fread(p, sizeof(*p), 1, filep);
+        (void)!fread(p, sizeof(*p), 1, filep);
     } else {
-        fscanf(filep, "%lf", p);
+        (void)!fscanf(filep, "%lf", p);
     }
 }
 
@@ -2351,7 +2459,7 @@ LFORTRAN_API void _lfortran_formatted_read(int32_t unit_num, int32_t* iostat, ch
     va_end(args);
 }
 
-LFORTRAN_API void _lfortran_empty_read(int32_t unit_num) {
+LFORTRAN_API void _lfortran_empty_read(int32_t unit_num, int32_t* iostat) {
     bool unit_file_bin;
     FILE* fp = get_file_pointer_from_unit(unit_num, &unit_file_bin);
     if (!fp) {
@@ -2365,6 +2473,14 @@ LFORTRAN_API void _lfortran_empty_read(int32_t unit_num) {
         while (c != '\n' && c != EOF) {
             c = fgetc(fp);
         }
+
+        if (feof(fp)) {
+            *iostat = -1;
+        } else if (ferror(fp)) {
+            *iostat = 1;
+        } else {
+            *iostat = 0;
+        }
     }
 }
 
@@ -2396,6 +2512,8 @@ LFORTRAN_API void _lfortran_file_write(int32_t unit_num, const char *format, ...
     va_start(args, format);
     vfprintf(filep, format, args);
     va_end(args);
+
+    (void)!ftruncate(fileno(filep), ftell(filep));
 }
 
 LFORTRAN_API void _lfortran_string_write(char **str, const char *format, ...) {
